#!/bin/bash

## MAIN VARIABLES ##
BEROCONF=/usr/fallback/beroconf

SETENV=/usr/fallback/fw_setenv
PRINTENV=/usr/fallback/fw_printenv

## EXIT FUNCTION ##
function rebootOnIssue() {
	## reboot in normal mode
	mkdir -p /usr/conf/permlog
	echo "[reproduce] [$(/bin/date '+%c')] ${1}" >> /usr/conf/permlog/reproduce.log
	${BEROCONF} set root reproduce-failed REPRODUCE_FAILED
	${BEROCONF} delete root boot_recoverymode
	/sbin/reboot &>/dev/null
	exit 1	
}

## MD5SUM FUNCTION ##
[ ! -f /tmp/reproduce.md5sum ] && rebootOnIssue "md5sum file missing"
source /tmp/reproduce.md5sum

function checkFileIntegrity() {
	case ${1} in
		app.jffs2)
			MD5SUM=${MD5SUM_APPFS}
			;;
		conf.jffs2)
			MD5SUM=${MD5SUM_CONFFS}
			;;
		root.jffs2)
			MD5SUM=${MD5SUM_ROOTFS}
			;;
		userapps.jffs2)
			MD5SUM=${MD5SUM_USERAPPFS}
			;;
		*)
			rebootOnIssue "Unknown file"
			;;
	esac
	if [ ! "$(/usr/bin/md5sum ${1} | /usr/bin/awk '{ print $1; }')" = "${MD5SUM}" ]; then
		rebootOnIssue "md5sum does not match (file=${1}|downloaded=$(/usr/bin/md5sum ${1} | /usr/bin/awk '{ print $1; }')|required=${MD5SUM})"
	fi
}

## UPDATE U-BOOT FUNCTION ##
function updateUboot() {
	if ! ${SETENV} ${1} &>/dev/null; then
		rebootOnIssue ${2}
	fi	
}

## CHECK INPUT ##
LOCALIP=${1}
[ -z ${LOCALIP} ] && rebootOnIssue "Usage: ${0} [localIp] [remoteIp] [remotePath]"

REMOTEIP=${2}
[ -z ${REMOTEIP} ] && rebootOnIssue "Missing Remote IP-Address"

REMOTEPATH=${3}
[ -z ${REMOTEPATH} ] && rebootOnIssue "Missing Remote Path"

## CHECK REQUIRED FILES ##
[ ! -f ${SETENV} ] && rebootOnIssue "'fw_setenv' file missing"
[ ! -f ${PRINTENV} ] && rebootOnIssue "'fw_printenv' file missing"

## CHECK TFTP-CONNECTION ##
TFTP=$(/usr/bin/which tftp)
[ -z ${TFTP} ] && rebootOnIssue "Missing tftp binary"

if ! /bin/ping -c 1 ${REMOTEIP} &>/dev/null; then
	rebootOnIssue "TFTP Server unavailable"
elif ! echo "quit" | ${TFTP} ${REMOTEIP} &>/dev/null; then
	rebootOnIssue "TFTP binary unstable / not working properly"
fi

## SET MTD1 ##
/bin/mount -o remount,rw /
if [ -c /dev/mtd1 ]; then
	if ! /bin/rm -f /dev/mtd1 &>/dev/null; then
		rebootOnIssue "Removing /dev/mtd1 failed"
	fi
fi
if ! /bin/mknod /dev/mtd1 c 90 2 &>/dev/null; then
	rebootOnIssue "Making /dev/mtd1 failed"
fi
/bin/mount -o remount,ro /

## CHECK FILES ON TFTP SERVER ##
cd /tmp
for file in "root.jffs2" "app.jffs2" "userapps.jffs2" "conf.jffs2"; do
	if ! echo "get ${REMOTEPATH}/${file}" | ${TFTP} ${REMOTEIP} &>/dev/null; then
		rebootOnIssue "Downloading '${file}' file failed"
	elif [ ! -f /tmp/${file} ]; then
		rebootOnIssue "'${file}' file missing"
	fi
	## check file integrity
	checkFileIntegrity ${file}
	rm -f /tmp/${file}
done

## MAIN ##
PREPPHY='run prep_phy; '
if [ ! ${PRINTENV} | /bin/grep 'prep_phy' &>/dev/null ]; then
	PREPPHY=''
fi

## fetch ssh-password
PASSWORD=$(/bin/cat /etc/shadow | /bin/grep root | /usr/bin/cut -d ':' -f 2)
updateUboot "password ${PASSWORD}" "Storing current password failed"

## update u-boot
updateUboot "ipaddr ${LOCALIP}" "Updating the Local IP-Address failed"
updateUboot "serverip ${REMOTEIP}" "Updating the Server IP-Address failed"
updateUboot "inst3 erase 1:35-86; tftp 1000000 ${REMOTEPATH}/root.jffs2; cp.b 1000000 11460000 \${filesize}" "Updating remote path failed (root.jffs2)"
updateUboot "inst4 erase 1:87-162; tftp 1000000 ${REMOTEPATH}/app.jffs2; cp.b 1000000 11a60000 \${filesize}" "Updating remote path failed (app.jffs2)"
updateUboot "inst5 erase 1:163-226; tftp 1000000 ${REMOTEPATH}/userapps.jffs2; cp.b 1000000 12460000 \${filesize}" "Updating remote path failed (userapps.jffs2)"
updateUboot "inst6 erase 1:227-254; tftp 1000000 ${REMOTEPATH}/conf.jffs2; cp.b 1000000 12c60000 \${filesize}" "Updating remote path failed (conf.jffs2)"
updateUboot "boot_ins ${PREPPHY}run inst0; run inst3; run inst4; run inst5; run inst6; run inst7; run inst8; run inst9" "Updating boot_ins failed"

## last update
updateUboot "reproduced 1" "Setting reproduced flag failed"
updateUboot "bootcmd run boot_ins" "Updating bootcmd command failed"

/sbin/reboot &>/dev/null
exit 0
